@@ -1,13 +1,16 @@
 # flake8: noqa
 from pkg_resources import DistributionNotFound, get_distribution
 
+from .core import resampling
 from .core.accessor import XSkillScoreAccessor
-from .core.comparative import sign_test
+from .core.comparative import halfwidth_ci_test, sign_test
 from .core.contingency import Contingency
 from .core.deterministic import (
     effective_sample_size,
+    linslope,
     mae,
     mape,
+    me,
     median_absolute_error,
     mse,
     pearson_r,
@@ -28,9 +31,11 @@ from .core.probabilistic import (
     discrimination,
     rank_histogram,
     reliability,
+    roc,
     rps,
     threshold_brier_score,
 )
+from .core.resampling import resample_iterations, resample_iterations_idx
 from .versioning.print_versions import show_versions
 
 try:
