@@ -1,10 +1,11 @@
-import numpy as np
 import xarray as xr
 
 from .deterministic import (
     effective_sample_size,
+    linslope,
     mae,
     mape,
+    me,
     median_absolute_error,
     mse,
     pearson_r,
@@ -25,12 +26,13 @@ from .probabilistic import (
     discrimination,
     rank_histogram,
     reliability,
+    roc,
     rps,
     threshold_brier_score,
 )
 
 
-@xr.register_dataset_accessor('xs')
+@xr.register_dataset_accessor("xs")
 class XSkillScoreAccessor(object):
     def __init__(self, xarray_obj):
         self._obj = xarray_obj
@@ -45,6 +47,11 @@ class XSkillScoreAccessor(object):
         else:
             return self._obj[x]
 
+    def linslope(self, a, b, *args, **kwargs):
+        a = self._in_ds(a)
+        b = self._in_ds(b)
+        return linslope(a, b, *args, **kwargs)
+
     def pearson_r(self, a, b, *args, **kwargs):
         a = self._in_ds(a)
         b = self._in_ds(b)
@@ -85,6 +92,11 @@ class XSkillScoreAccessor(object):
         b = self._in_ds(b)
         return spearman_r_eff_p_value(a, b, *args, **kwargs)
 
+    def me(self, a, b, *args, **kwargs):
+        a = self._in_ds(a)
+        b = self._in_ds(b)
+        return me(a, b, *args, **kwargs)
+
     def rmse(self, a, b, *args, **kwargs):
         a = self._in_ds(a)
         b = self._in_ds(b)
@@ -164,3 +176,8 @@ class XSkillScoreAccessor(object):
         observations = self._in_ds(observations)
         forecasts = self._in_ds(forecasts)
         return reliability(observations, forecasts, *args, **kwargs)
+
+    def roc(self, observations, forecasts, *args, **kwargs):
+        observations = self._in_ds(observations)
+        forecasts = self._in_ds(forecasts)
+        return roc(observations, forecasts, *args, **kwargs)
