@@ -3,19 +3,25 @@ import numpy as np
 from scipy import special
 from scipy.stats import distributions
 
+from .utils import suppress_warnings
+
 __all__ = [
-    '_pearson_r',
-    '_pearson_r_p_value',
-    '_rmse',
-    '_mse',
-    '_mae',
-    '_median_absolute_error',
-    '_smape',
-    '_mape',
-    '_spearman_r',
-    '_spearman_r_p_value',
-    '_effective_sample_size',
-    '_r2',
+    "_effective_sample_size",
+    "_linslope",
+    "_mae",
+    "_mape",
+    "_me",
+    "_median_absolute_error",
+    "_mse",
+    "_pearson_r",
+    "_pearson_r_eff_p_value",
+    "_pearson_r_p_value",
+    "_r2",
+    "_rmse",
+    "_smape",
+    "_spearman_r",
+    "_spearman_r_eff_p_value",
+    "_spearman_r_p_value",
 ]
 
 
@@ -38,7 +44,7 @@ def _match_nans(a, b, weights):
         # Find pairwise indices in a and b that have nans.
         idx = np.logical_or(np.isnan(a), np.isnan(b))
         a[idx], b[idx] = np.nan, np.nan
-        # https://github.com/raybellwaves/xskillscore/issues/168
+        # https://github.com/xarray-contrib/xskillscore/issues/168
         if isinstance(weights, np.ndarray):
             if weights.shape:  # not None
                 weights = weights.copy()
@@ -70,6 +76,11 @@ def _check_weights(weights):
     elif np.all(np.isnan(weights)):
         return None
     else:
+        if weights.min() < 0:
+            raise ValueError(
+                "Weights has a minimum below 0. Please submit a weights array "
+                "of positive numbers."
+            )
         return weights
 
 
@@ -79,11 +90,13 @@ def __compute_anomalies(a, b, weights, axis, skipna):
     # single generic function with weights of all ones, because
     # the denominator gets inflated when there are masked regions.
     if weights is not None:
-        ma = sumfunc(a * weights, axis=axis) / sumfunc(weights, axis=axis)
-        mb = sumfunc(b * weights, axis=axis) / sumfunc(weights, axis=axis)
+        with suppress_warnings("invalid value encountered in true_divide"):
+            ma = sumfunc(a * weights, axis=axis) / sumfunc(weights, axis=axis)
+            mb = sumfunc(b * weights, axis=axis) / sumfunc(weights, axis=axis)
     else:
-        ma = meanfunc(a, axis=axis)
-        mb = meanfunc(b, axis=axis)
+        with suppress_warnings("Mean of empty slice"):
+            ma = meanfunc(a, axis=axis)
+            mb = meanfunc(b, axis=axis)
     am, bm = a - ma, b - mb
     return am, bm
 
@@ -134,14 +147,15 @@ def _effective_sample_size(a, b, axis, skipna):
     b_auto = _pearson_r(bm[0:-1], bm[1::], weights=None, axis=0, skipna=skipna)
 
     # compute effective sample size per Bretherton et al. 1999
-    n_eff = n * (1 - a_auto * b_auto) / (1 + a_auto * b_auto)
+    with suppress_warnings("divide by zero encountered in true_divide"):
+        n_eff = n * (1 - a_auto * b_auto) / (1 + a_auto * b_auto)
     n_eff = np.floor(n_eff)
     n_eff = np.clip(n_eff, 0, n)
     return n_eff
 
 
-def _pearson_r(a, b, weights, axis, skipna):
-    """ndarray implementation of scipy.stats.pearsonr.
+def _linslope(a, b, weights, axis, skipna):
+    """ndarray implementation of scipy.stats.linregress[slope].
 
     Parameters
     ----------
@@ -150,7 +164,7 @@ def _pearson_r(a, b, weights, axis, skipna):
     b : ndarray
         Input array.
     axis : int
-        The axis to apply the correlation along.
+        The axis to apply the linear slope along.
     weights : ndarray
         Input array of weights for a and b.
     skipna : bool
@@ -159,11 +173,11 @@ def _pearson_r(a, b, weights, axis, skipna):
     Returns
     -------
     res : ndarray
-        Pearson's correlation coefficient.
+        slope of linear fit.
 
     See Also
     --------
-    scipy.stats.pearsonr
+    scipy.stats.linregress
     """
     sumfunc, meanfunc = _get_numpy_funcs(skipna)
     if skipna:
@@ -177,16 +191,15 @@ def _pearson_r(a, b, weights, axis, skipna):
     am, bm = __compute_anomalies(a, b, weights=weights, axis=0, skipna=skipna)
 
     if weights is not None:
-        r_num = sumfunc(weights * am * bm, axis=0)
-        r_den = np.sqrt(
-            sumfunc(weights * am * am, axis=0) * sumfunc(weights * bm * bm, axis=0)
-        )
+        s_num = sumfunc(weights * am * bm, axis=0)
+        s_den = sumfunc(weights * am * am, axis=0)
     else:
-        r_num = sumfunc(am * bm, axis=0)
-        r_den = np.sqrt(sumfunc(am * am, axis=0) * sumfunc(bm * bm, axis=0))
+        s_num = sumfunc(am * bm, axis=0)
+        s_den = sumfunc(am * am, axis=0)
 
-    r = r_num / r_den
-    res = np.clip(r, -1.0, 1.0)
+    with suppress_warnings("invalid value encountered in true_divide"):
+        with suppress_warnings("invalid value encountered in double_scalars"):
+            res = s_num / s_den
     return res
 
 
@@ -238,10 +251,66 @@ def _r2(a, b, weights, axis, skipna):
         am_squared = am ** 2
     num = sumfunc(squared_error, axis=0)
     den = sumfunc(am_squared, axis=0)
-    r2 = 1 - (num / den)
+    with suppress_warnings("invalid value encountered in true_divide"):
+        with suppress_warnings("divide by zero encountered in true_divide"):
+            with suppress_warnings("divide by zero encountered in double_scalars"):
+                with suppress_warnings("invalid value encountered in double_scalars"):
+                    r2 = 1 - (num / den)
     return r2
 
 
+def _pearson_r(a, b, weights, axis, skipna):
+    """ndarray implementation of scipy.stats.pearsonr.
+
+    Parameters
+    ----------
+    a : ndarray
+        Input array.
+    b : ndarray
+        Input array.
+    axis : int
+        The axis to apply the correlation along.
+    weights : ndarray
+        Input array of weights for a and b.
+    skipna : bool
+        If True, skip NaNs when computing function.
+
+    Returns
+    -------
+    res : ndarray
+        Pearson's correlation coefficient.
+
+    See Also
+    --------
+    scipy.stats.pearsonr
+    """
+    sumfunc, meanfunc = _get_numpy_funcs(skipna)
+    if skipna:
+        a, b, weights = _match_nans(a, b, weights)
+    weights = _check_weights(weights)
+    a = np.rollaxis(a, axis)
+    b = np.rollaxis(b, axis)
+    if weights is not None:
+        weights = np.rollaxis(weights, axis)
+
+    am, bm = __compute_anomalies(a, b, weights=weights, axis=0, skipna=skipna)
+
+    if weights is not None:
+        r_num = sumfunc(weights * am * bm, axis=0)
+        r_den = np.sqrt(
+            sumfunc(weights * am * am, axis=0) * sumfunc(weights * bm * bm, axis=0)
+        )
+    else:
+        r_num = sumfunc(am * bm, axis=0)
+        r_den = np.sqrt(sumfunc(am * am, axis=0) * sumfunc(bm * bm, axis=0))
+
+    with suppress_warnings("invalid value encountered in true_divide"):
+        with suppress_warnings("invalid value encountered in double_scalars"):
+            r = r_num / r_den
+    res = np.clip(r, -1.0, 1.0)
+    return res
+
+
 def _pearson_r_p_value(a, b, weights, axis, skipna):
     """ndarray implementation of scipy.stats.pearsonr.
 
@@ -278,14 +347,19 @@ def _pearson_r_p_value(a, b, weights, axis, skipna):
         b = np.rollaxis(b, axis)
         # count non-nans
         dof = np.count_nonzero(~np.isnan(a), axis=0) - 2
-        t_squared = r ** 2 * (dof / ((1.0 - r) * (1.0 + r)))
-        _x = dof / (dof + t_squared)
+        with suppress_warnings("invalid value encountered in true_divide"):
+            t_squared = r ** 2 * (dof / ((1.0 - r) * (1.0 + r)))
+            _x = dof / (dof + t_squared)
         _x = np.asarray(_x)
         _x = np.where(_x < 1.0, _x, 1.0)
         _a = 0.5 * dof
         _b = 0.5
         res = special.betainc(_a, _b, _x)
         # reset masked values to nan
+        # raises  <__array_function__ internals>:5: DeprecationWarning: Calling nonzero
+        # on 0d arrays is deprecated, as it behaves surprisingly. Use
+        # `atleast_1d(cond).nonzero()` if the old behavior was intended. If the context
+        # of this warning is of the form `arr[nonzero(cond)]`, just use `arr[cond]`.
         nan_locs = np.where(np.isnan(r))
         if len(nan_locs[0]) > 0:
             res[nan_locs] = np.nan
@@ -332,7 +406,8 @@ def _pearson_r_eff_p_value(a, b, axis, skipna):
     else:
         dof = _effective_sample_size(a, b, axis, skipna) - 2
         t_squared = r ** 2 * (dof / ((1.0 - r) * (1.0 + r)))
-        _x = dof / (dof + t_squared)
+        with suppress_warnings("invalid value encountered in true_divide"):
+            _x = dof / (dof + t_squared)
         _x = np.asarray(_x)
         _x = np.where(_x < 1.0, _x, 1.0)
         _a = 0.5 * dof
@@ -372,9 +447,8 @@ def _spearman_r(a, b, weights, axis, skipna):
     """
     if skipna:
         a, b, weights = _match_nans(a, b, weights)
-    rankfunc = bn.nanrankdata
-    _a = rankfunc(a, axis=axis)
-    _b = rankfunc(b, axis=axis)
+    _a = bn.nanrankdata(a, axis=axis)
+    _b = bn.nanrankdata(b, axis=axis)
     return _pearson_r(_a, _b, weights, axis, skipna)
 
 
@@ -414,7 +488,9 @@ def _spearman_r_p_value(a, b, weights, axis, skipna):
     b = np.rollaxis(b, axis)
     # count non-nans
     dof = np.count_nonzero(~np.isnan(a), axis=0) - 2
-    t = rs * np.sqrt((dof / ((rs + 1.0) * (1.0 - rs))).clip(0))
+    with suppress_warnings("invalid value encountered in true_divide"):
+        with suppress_warnings("divide by zero encountered in true_divide"):
+            t = rs * np.sqrt((dof / ((rs + 1.0) * (1.0 - rs))).clip(0))
     p = 2 * distributions.t.sf(np.abs(t), dof)
     return p
 
@@ -459,11 +535,51 @@ def _spearman_r_eff_p_value(a, b, axis, skipna):
         a, b, _ = _match_nans(a, b, None)
     rs = _spearman_r(a, b, None, axis, skipna)
     dof = _effective_sample_size(a, b, axis, skipna) - 2
-    t = rs * np.sqrt((dof / ((rs + 1.0) * (1.0 - rs))).clip(0))
+    with suppress_warnings("invalid value encountered in true_divide"):
+        with suppress_warnings("divide by zero encountered in true_divide"):
+            t = rs * np.sqrt((dof / ((rs + 1.0) * (1.0 - rs))).clip(0))
     p = 2 * distributions.t.sf(np.abs(t), dof)
     return p
 
 
+def _me(a, b, weights, axis, skipna):
+    """Mean Error.
+
+    Parameters
+    ----------
+    a : ndarray
+        Input array.
+    b : ndarray
+        Input array.
+    axis : int
+        The axis to apply the me along.
+    weights : ndarray
+        Input array of weights for a and b.
+    skipna : bool
+        If True, skip NaNs when computing function.
+
+    Returns
+    -------
+    res : ndarray
+        Mean Error.
+    """
+    sumfunc, meanfunc = _get_numpy_funcs(skipna)
+    if skipna:
+        a, b, weights = _match_nans(a, b, weights)
+    weights = _check_weights(weights)
+
+    error = a - b
+    if weights is not None:
+        with suppress_warnings("invalid value encountered in true_divide"):
+            mean_error = sumfunc(error * weights, axis=axis) / sumfunc(
+                weights, axis=axis
+            )
+    else:
+        with suppress_warnings("Mean of empty slice"):
+            mean_error = meanfunc(error, axis=axis)
+    return mean_error
+
+
 def _rmse(a, b, weights, axis, skipna):
     """Root Mean Squared Error.
 
@@ -496,11 +612,13 @@ def _rmse(a, b, weights, axis, skipna):
 
     squared_error = (a - b) ** 2
     if weights is not None:
-        mean_squared_error = sumfunc(squared_error * weights, axis=axis) / sumfunc(
-            weights, axis=axis
-        )
+        with suppress_warnings("invalid value encountered in true_divide"):
+            mean_squared_error = sumfunc(squared_error * weights, axis=axis) / sumfunc(
+                weights, axis=axis
+            )
     else:
-        mean_squared_error = meanfunc(squared_error, axis=axis)
+        with suppress_warnings("Mean of empty slice"):
+            mean_squared_error = meanfunc(squared_error, axis=axis)
     res = np.sqrt(mean_squared_error)
     return res
 
@@ -537,9 +655,13 @@ def _mse(a, b, weights, axis, skipna):
 
     squared_error = (a - b) ** 2
     if weights is not None:
-        return sumfunc(squared_error * weights, axis=axis) / sumfunc(weights, axis=axis)
+        with suppress_warnings("invalid value encountered in true_divide"):
+            return sumfunc(squared_error * weights, axis=axis) / sumfunc(
+                weights, axis=axis
+            )
     else:
-        return meanfunc(squared_error, axis=axis)
+        with suppress_warnings("Mean of empty slice"):
+            return meanfunc(squared_error, axis=axis)
 
 
 def _mae(a, b, weights, axis, skipna):
@@ -574,11 +696,13 @@ def _mae(a, b, weights, axis, skipna):
 
     absolute_error = np.absolute(a - b)
     if weights is not None:
-        return sumfunc(absolute_error * weights, axis=axis) / sumfunc(
-            weights, axis=axis
-        )
+        with suppress_warnings("invalid value encountered in true_divide"):
+            return sumfunc(absolute_error * weights, axis=axis) / sumfunc(
+                weights, axis=axis
+            )
     else:
-        return meanfunc(absolute_error, axis=axis)
+        with suppress_warnings("Mean of empty slice"):
+            return meanfunc(absolute_error, axis=axis)
 
 
 def _median_absolute_error(a, b, axis, skipna):
@@ -608,7 +732,8 @@ def _median_absolute_error(a, b, axis, skipna):
     if skipna:
         a, b, _ = _match_nans(a, b, None)
     absolute_error = np.absolute(a - b)
-    return medianfunc(absolute_error, axis=axis)
+    with suppress_warnings("All-NaN slice encountered"):
+        return medianfunc(absolute_error, axis=axis)
 
 
 def _mape(a, b, weights, axis, skipna):
@@ -617,7 +742,7 @@ def _mape(a, b, weights, axis, skipna):
     .. math::
         \\mathrm{MAPE} = \\frac{1}{n} \\sum_{i=1}^{n}
                          \\frac{\\vert a_{i} - b_{i} \\vert}
-                               {\\vert a_{i} \\vert}
+                               {max(\epsilon, \\vert a_{i} \\vert)}
 
     Parameters
     ----------
@@ -644,6 +769,13 @@ def _mape(a, b, weights, axis, skipna):
     Percent error is reported as decimal percent. I.e., a value of
     1 is 100%.
 
+    \epsilon is an arbitrary small yet strictly positive number to avoid
+    undefined results when ``a`` is zero.
+
+    See Also
+    --------
+    sklearn.metrics.mean_absolute_percentage_error
+
     References
     ----------
     https://en.wikipedia.org/wiki/Mean_absolute_percentage_error
@@ -652,12 +784,14 @@ def _mape(a, b, weights, axis, skipna):
     if skipna:
         a, b, weights = _match_nans(a, b, weights)
     weights = _check_weights(weights)
-    # replace divided by 0 with nan
-    mape = np.absolute(a - b) / np.absolute(np.where(a != 0, a, np.nan))
+    epsilon = np.finfo(np.float64).eps
+    mape = np.absolute(a - b) / np.maximum(np.absolute(a), epsilon)
     if weights is not None:
-        return sumfunc(mape * weights, axis=axis) / sumfunc(weights, axis=axis)
+        with suppress_warnings("invalid value encountered in true_divide"):
+            return sumfunc(mape * weights, axis=axis) / sumfunc(weights, axis=axis)
     else:
-        return meanfunc(mape, axis=axis)
+        with suppress_warnings("Mean of empty slice"):
+            return meanfunc(mape, axis=axis)
 
 
 def _smape(a, b, weights, axis, skipna):
@@ -701,6 +835,8 @@ def _smape(a, b, weights, axis, skipna):
     weights = _check_weights(weights)
     smape = np.absolute(a - b) / (np.absolute(a) + np.absolute(b))
     if weights is not None:
-        return sumfunc(smape * weights, axis=axis) / sumfunc(weights, axis=axis)
+        with suppress_warnings("invalid value encountered in true_divide"):
+            return sumfunc(smape * weights, axis=axis) / sumfunc(weights, axis=axis)
     else:
-        return meanfunc(smape, axis=axis)
+        with suppress_warnings("Mean of empty slice"):
+            return meanfunc(smape, axis=axis)
